"""
PearProject - `Syracuse`
~~~~~~~~~~~~~~~~~~~
Module sur la suite de Syracuse, en imaginant que les valeurs de la suite correspond à la hauteur d'un avion en papier en vol.
"""

from matplotlib import pyplot
from .errors import *
from .decorators import *

class syracuse_functions():
    def next_step(int_number: int):
        """Calcule la prochaine étape d'un nombre dans la suite de Syracuse.

        Args:
            int_number (int): Nombre entier supérieur à 1.

        Raises:
            TooSmall: Si le nombre est inférieur ou égal à 1.

        Returns:
            [int]: Retourne un nombre entier supérieur à 1.
        """
        if int_number <= 1:
            raise TooSmall(1)
        if ((int_number % 2) == 0):
            int_number = int_number/2
        else:
            int_number = int_number*3+1
        return int(int_number)

    def flight_time(int_list: list):
        """Calcule le temps de vol.

        Args:
            int_list (list): Liste contenant plus d'un élement et uniquement des nombres entiers.

        Raises:
            IncorrectListItemType: Si la liste ne contient pas uniqument des nombres entier.
            ListTooSmall: Si la liste contient moins de 2 éléments.

        Returns:
            [int]: Retourne un nombre entier supérieur ou égal à 1.
        """
        for i in int_list:
            if type(i) != int:
                raise IncorrectListItemType
        if len(int_list) <= 1:
            raise ListTooSmall(1)
        return len(int_list)-1

    def max_flight_altitude(int_list: list):
        """Détermine l'altitude maximale de la suite.

        Args:
            int_list (list): Liste contenant plus d'un élement et uniquement des nombres entiers.

        Raises:
            IncorrectListItemType: Si la liste ne contient pas uniqument des nombres entier.
            ListTooSmall: Si la liste contient moins de 2 éléments.

        Returns:
            [int]: Retourne un nombre entier.
        """
        for i in int_list:
            if type(i) != int:
                raise IncorrectListItemType
        if len(int_list) <= 1:
            raise TooSmall(1)
        return max(int_list)

    def time_flight_altitude(int_list: list):
        """Détermine le nombre d'étapes nécesaire pour descendre en dessous de l'atiltude initial.

        Args:
            int_list (list): Liste contenant plus d'un élement et uniquement des nombres entiers.

        Raises:
            IncorrectListItemType: Si la liste ne contient pas uniqument des nombres entier.
            ListTooSmall: Si la liste contient moins de 2 éléments.

        Returns:
            [int]: Retourne un nombre entier.
        """
        for i in int_list:
            if type(i) != int:
                raise IncorrectListItemType
        if len(int_list) <= 1:
            raise TooSmall(1)
        top = int_list[0]
        for i in int_list:
            if i < top:
                return (int_list.index(i)-1)
        return None

    def syracuse_graphic(x_list: list):
        """Créer un graphique sur une suite de syracuse.

        Args:
            x_list (list): Liste contenant plus d'un élement et uniquement des nombres entiers.

        Raises:
            IncorrectListItemType: Si la liste ne contient pas uniqument des nombres entier.
            ListTooSmall: Si la liste contient moins de 2 éléments.

        Returns:
            [NoneType]: Retourne rien.
        """
        for i in x_list:
            if type(i) != int:
                raise IncorrectListItemType
        if len(x_list) <= 1:
            raise TooSmall(1)
        pyplot.axis([0, len(x_list)-1, min(x_list), max(x_list)+1])
        pyplot.plot(range(len(x_list)), x_list, "b*--")
        pyplot.xlabel("Temps")
        pyplot.ylabel("Hauteur")
        pyplot.title("Graphique de la suite de Syracuse")
        pyplot.show()
        return None

@type_verifier
def syracuse(int_number: int):
    """Calcule et retourne une liste créée avec les termes de la suite de Syracuse de l'entier initial supérieur à 1 jusqu'à 1 inclus.

    Args:
        int_number (int): Nombre entier supérieur à 1.

    Raises:
        TooSmall: Si le nombre est inférieur ou égal à 1.

    Returns:
        [list]: Retourne une liste de nombres entier supérieur à 1.
    """
    if int_number <= 0:
        raise TooSmall(1)
    flight_list = [int_number]
    while int_number != 1:
        int_number = syracuse_functions.next_step(int_number)
        flight_list.append(int_number)
    return flight_list

@type_verifier
def syracuse_info(int_number: int, graphic: bool = True):
    """Calcule la suite de Syracuse selon un entier de départ supérieur à 1 et donne des informations complémentaires.

    Args:
        int_number (int): Nombre entier supérieur à 1.
        graphic (bool, optionnel): Si True affiche un graphique à la fin, Si False ne l'affiche pas. True par default.

    Raises:
        TooSmall: Si le nombre est inférieur ou égal à 1.

    Returns:
        [NoneType]: Retourne rien.
    """
    if int_number <= 1:
        raise TooSmall(1)
    flight = syracuse(int_number)
    flight_time = syracuse_functions.flight_time(flight)
    max_flight_altitude = syracuse_functions.max_flight_altitude(flight)
    time_flight_altitude = syracuse_functions.time_flight_altitude(flight)
    flight_str = ", ".join([str(i) for i in flight])
    print(f"La suite de Syracuse: {flight_str}\n"
          f"Le temps de vol: {flight_time}\n"
          f"L'altitude maximal: {max_flight_altitude}\n"
          f"Le temps avant avant de redescendre: {time_flight_altitude}\n"
          )
    if graphic == True:
        syracuse_functions.syracuse_graphic(flight)
    return None
